@@ -1,10 +1,11 @@
-use crate::common::color::Color;
+use crate::common::color::{Color, ColorWrapper};
 use crate::common::{
     Anchor, Calendar, ColorBar, ColorScale, DashType, Font, Label, Orientation, Side,
-    TickFormatStops, TickMode, Title,
+    TickFormatStop, TickMode, Title,
 };
 use crate::plot::Trace;
 use crate::private;
+use crate::private::{to_num_or_string_wrapper, NumOrString, NumOrStringWrapper, TruthyEnum};
 use serde::Serialize;
 
 #[derive(Serialize, Debug)]
@@ -18,7 +19,7 @@ pub enum AxisType {
     #[serde(rename = "date")]
     Date,
     #[serde(rename = "category")]
-    Cetegory,
+    Category,
     #[serde(rename = "multicategory")]
     MultiCategory,
 }
@@ -131,12 +132,12 @@ pub enum WaterfallMode {
     Overlay,
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct Legend {
     #[serde(skip_serializing_if = "Option::is_none", rename = "bgcolor")]
-    background_color: Option<String>,
+    background_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "bordercolor")]
-    border_color: Option<String>,
+    border_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "borderwidth")]
     border_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -169,33 +170,16 @@ pub struct Legend {
 
 impl Legend {
     pub fn new() -> Legend {
-        Legend {
-            background_color: None,
-            border_color: None,
-            border_width: None,
-            font: None,
-            orientation: None,
-            trace_order: None,
-            trace_group_gap: None,
-            item_sizing: None,
-            item_click: None,
-            item_double_click: None,
-            x: None,
-            x_anchor: None,
-            y: None,
-            y_anchor: None,
-            valign: None,
-            title: None,
-        }
+        Default::default()
     }
 
     pub fn background_color<C: Color>(mut self, background_color: C) -> Legend {
-        self.background_color = Some(background_color.to_color_string());
+        self.background_color = Some(background_color.to_color());
         self
     }
 
     pub fn border_color<C: Color>(mut self, border_color: C) -> Legend {
-        self.border_color = Some(border_color.to_color_string());
+        self.border_color = Some(border_color.to_color());
         self
     }
 
@@ -280,7 +264,7 @@ pub enum Align {
     Bottom,
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct Margin {
     #[serde(skip_serializing_if = "Option::is_none")]
     l: Option<usize>,
@@ -298,14 +282,7 @@ pub struct Margin {
 
 impl Margin {
     pub fn new() -> Margin {
-        Margin {
-            l: None,
-            r: None,
-            t: None,
-            b: None,
-            pad: None,
-            auto_expand: None,
-        }
+        Default::default()
     }
 
     pub fn left(mut self, left: usize) -> Margin {
@@ -339,7 +316,7 @@ impl Margin {
     }
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct LayoutColorScale {
     #[serde(skip_serializing_if = "Option::is_none")]
     sequential: Option<ColorScale>,
@@ -351,11 +328,7 @@ pub struct LayoutColorScale {
 
 impl LayoutColorScale {
     pub fn new() -> LayoutColorScale {
-        LayoutColorScale {
-            sequential: None,
-            sequential_minus: None,
-            diverging: None,
-        }
+        Default::default()
     }
 
     pub fn sequential(mut self, sequential: ColorScale) -> LayoutColorScale {
@@ -375,6 +348,280 @@ impl LayoutColorScale {
 }
 
 #[derive(Serialize, Debug)]
+pub enum SliderRangeMode {
+    #[serde(rename = "auto")]
+    Auto,
+    #[serde(rename = "fixed")]
+    Fixed,
+    #[serde(rename = "match")]
+    Match,
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct RangeSliderYAxis {
+    #[serde(skip_serializing_if = "Option::is_none", rename = "rangemode")]
+    range_mode: Option<SliderRangeMode>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    range: Option<Vec<NumOrStringWrapper>>,
+}
+
+impl RangeSliderYAxis {
+    pub fn new() -> RangeSliderYAxis {
+        Default::default()
+    }
+
+    pub fn range_mode(mut self, range_mode: SliderRangeMode) -> RangeSliderYAxis {
+        self.range_mode = Some(range_mode);
+        self
+    }
+
+    pub fn range<C: NumOrString>(mut self, range: Vec<C>) -> RangeSliderYAxis {
+        let wrapped = to_num_or_string_wrapper(range);
+        self.range = Some(wrapped);
+        self
+    }
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct RangeSlider {
+    #[serde(skip_serializing_if = "Option::is_none", rename = "bgcolor")]
+    background_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "bordercolor")]
+    border_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "borderwidth")]
+    border_width: Option<u64>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "autorange")]
+    auto_range: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    range: Option<Vec<NumOrStringWrapper>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    thickness: Option<f64>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    visible: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis")]
+    y_axis: Option<RangeSliderYAxis>,
+}
+
+impl RangeSlider {
+    pub fn new() -> RangeSlider {
+        Default::default()
+    }
+
+    pub fn background_color<C: Color>(mut self, background_color: C) -> RangeSlider {
+        self.background_color = Some(background_color.to_color());
+        self
+    }
+
+    pub fn border_color<C: Color>(mut self, border_color: C) -> RangeSlider {
+        self.border_color = Some(border_color.to_color());
+        self
+    }
+
+    pub fn border_width(mut self, border_width: u64) -> RangeSlider {
+        self.border_width = Some(border_width);
+        self
+    }
+
+    pub fn auto_range(mut self, auto_range: bool) -> RangeSlider {
+        self.auto_range = Some(auto_range);
+        self
+    }
+
+    pub fn range<C: NumOrString>(mut self, range: Vec<C>) -> RangeSlider {
+        let wrapped = to_num_or_string_wrapper(range);
+        self.range = Some(wrapped);
+        self
+    }
+
+    pub fn thickness(mut self, thickness: f64) -> RangeSlider {
+        self.thickness = Some(thickness);
+        self
+    }
+
+    pub fn visible(mut self, visible: bool) -> RangeSlider {
+        self.visible = Some(visible);
+        self
+    }
+
+    pub fn y_axis(mut self, axis: RangeSliderYAxis) -> RangeSlider {
+        self.y_axis = Some(axis);
+        self
+    }
+}
+
+#[derive(Serialize, Debug)]
+pub enum SelectorStep {
+    #[serde(rename = "month")]
+    Month,
+    #[serde(rename = "year")]
+    Year,
+    #[serde(rename = "day")]
+    Day,
+    #[serde(rename = "hour")]
+    Hour,
+    #[serde(rename = "minute")]
+    Minute,
+    #[serde(rename = "second")]
+    Second,
+    #[serde(rename = "all")]
+    All,
+}
+
+#[derive(Serialize, Debug)]
+pub enum StepMode {
+    #[serde(rename = "backward")]
+    Backward,
+    #[serde(rename = "todate")]
+    ToDate,
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct SelectorButton {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    visible: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    step: Option<SelectorStep>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "stepmode")]
+    step_mode: Option<StepMode>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    count: Option<usize>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    label: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    name: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "templateitemname")]
+    template_item_name: Option<String>,
+}
+
+impl SelectorButton {
+    pub fn new() -> SelectorButton {
+        Default::default()
+    }
+
+    pub fn visible(mut self, visible: bool) -> SelectorButton {
+        self.visible = Some(visible);
+        self
+    }
+
+    pub fn step(mut self, step: SelectorStep) -> SelectorButton {
+        self.step = Some(step);
+        self
+    }
+
+    pub fn step_mode(mut self, step_mode: StepMode) -> SelectorButton {
+        self.step_mode = Some(step_mode);
+        self
+    }
+
+    pub fn count(mut self, count: usize) -> SelectorButton {
+        self.count = Some(count);
+        self
+    }
+
+    pub fn label(mut self, label: &str) -> SelectorButton {
+        self.label = Some(label.to_owned());
+        self
+    }
+
+    pub fn name(mut self, name: &str) -> SelectorButton {
+        self.name = Some(name.to_owned());
+        self
+    }
+
+    pub fn template_item_name(mut self, template_item_name: &str) -> SelectorButton {
+        self.template_item_name = Some(template_item_name.to_owned());
+        self
+    }
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct RangeSelector {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    visible: Option<bool>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    buttons: Option<Vec<SelectorButton>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    x: Option<f64>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xanchor")]
+    x_anchor: Option<Anchor>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    y: Option<f64>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yanchor")]
+    y_anchor: Option<Anchor>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    font: Option<Font>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "bgcolor")]
+    background_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "activecolor")]
+    active_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "bordercolor")]
+    border_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "borderwidth")]
+    border_width: Option<usize>,
+}
+
+impl RangeSelector {
+    pub fn new() -> RangeSelector {
+        Default::default()
+    }
+
+    pub fn visible(mut self, visible: bool) -> RangeSelector {
+        self.visible = Some(visible);
+        self
+    }
+
+    pub fn buttons(mut self, buttons: Vec<SelectorButton>) -> RangeSelector {
+        self.buttons = Some(buttons);
+        self
+    }
+
+    pub fn x(mut self, x: f64) -> RangeSelector {
+        self.x = Some(x);
+        self
+    }
+
+    pub fn x_anchor(mut self, x_anchor: Anchor) -> RangeSelector {
+        self.x_anchor = Some(x_anchor);
+        self
+    }
+
+    pub fn y(mut self, y: f64) -> RangeSelector {
+        self.y = Some(y);
+        self
+    }
+
+    pub fn y_anchor(mut self, y_anchor: Anchor) -> RangeSelector {
+        self.y_anchor = Some(y_anchor);
+        self
+    }
+
+    pub fn font(mut self, font: Font) -> RangeSelector {
+        self.font = Some(font);
+        self
+    }
+
+    pub fn background_color<C: Color>(mut self, background_color: C) -> RangeSelector {
+        self.background_color = Some(background_color.to_color());
+        self
+    }
+
+    pub fn active_color<C: Color>(mut self, active_color: C) -> RangeSelector {
+        self.background_color = Some(active_color.to_color());
+        self
+    }
+
+    pub fn border_color<C: Color>(mut self, border_color: C) -> RangeSelector {
+        self.border_color = Some(border_color.to_color());
+        self
+    }
+
+    pub fn border_width(mut self, border_width: usize) -> RangeSelector {
+        self.border_width = Some(border_width);
+        self
+    }
+}
+
+#[derive(Serialize, Debug, Default)]
 pub struct ColorAxis {
     #[serde(skip_serializing_if = "Option::is_none")]
     cauto: Option<bool>,
@@ -398,17 +645,7 @@ pub struct ColorAxis {
 
 impl ColorAxis {
     pub fn new() -> ColorAxis {
-        ColorAxis {
-            cauto: None,
-            cmin: None,
-            cmax: None,
-            cmid: None,
-            color_scale: None,
-            auto_color_scale: None,
-            reverse_scale: None,
-            show_scale: None,
-            color_bar: None,
-        }
+        Default::default()
     }
 
     pub fn cauto(mut self, cauto: bool) -> ColorAxis {
@@ -457,12 +694,12 @@ impl ColorAxis {
     }
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct Axis {
     #[serde(skip_serializing_if = "Option::is_none")]
     visible: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    color: Option<String>,
+    color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none")]
     title: Option<Title>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -472,7 +709,7 @@ pub struct Axis {
     #[serde(skip_serializing_if = "Option::is_none", rename = "rangemode")]
     range_mode: Option<RangeMode>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    range: Option<Vec<f64>>,
+    range: Option<Vec<NumOrStringWrapper>>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "fixedrange")]
     fixed_range: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -504,7 +741,7 @@ pub struct Axis {
     #[serde(skip_serializing_if = "Option::is_none", rename = "tickwidth")]
     tick_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "tickcolor")]
-    tick_color: Option<String>,
+    tick_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "showticklabels")]
     show_tick_labels: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "automargin")]
@@ -512,7 +749,7 @@ pub struct Axis {
     #[serde(skip_serializing_if = "Option::is_none", rename = "showspikes")]
     show_spikes: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "spikecolor")]
-    spike_color: Option<String>,
+    spike_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "spikethickness")]
     spike_thickness: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "spikedash")]
@@ -542,105 +779,54 @@ pub struct Axis {
     #[serde(skip_serializing_if = "Option::is_none", rename = "tickformat")]
     tick_format: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "tickformatstops")]
-    tick_format_stops: Option<TickFormatStops>,
+    tick_format_stops: Option<Vec<TickFormatStop>>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "hoverformat")]
     hover_format: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "showline")]
     show_line: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "linecolor")]
-    line_color: Option<String>,
+    line_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "linewidth")]
     line_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "showgrid")]
     show_grid: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "gridcolor")]
-    grid_color: Option<String>,
+    grid_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "gridwidth")]
     grid_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "zeroline")]
     zero_line: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "zerolinecolor")]
-    zero_line_color: Option<String>,
+    zero_line_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "zerolinewidth")]
     zero_line_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "showdividers")]
     show_dividers: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "dividercolor")]
-    divider_color: Option<String>,
+    divider_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "dividerwidth")]
     divider_width: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none")]
+    anchor: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     side: Option<Side>,
     #[serde(skip_serializing_if = "Option::is_none")]
+    overlaying: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     domain: Option<Vec<f64>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     position: Option<f64>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "rangeslider")]
+    range_slider: Option<RangeSlider>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "rangeselector")]
+    range_selector: Option<RangeSelector>,
     #[serde(skip_serializing_if = "Option::is_none")]
     calendar: Option<Calendar>,
 }
 
 impl Axis {
     pub fn new() -> Axis {
-        Axis {
-            visible: None,
-            color: None,
-            title: None,
-            r#type: None,
-            auto_range: None,
-            range_mode: None,
-            range: None,
-            fixed_range: None,
-            constrain: None,
-            constrain_toward: None,
-            tick_mode: None,
-            n_ticks: None,
-            tick0: None,
-            dtick: None,
-            tick_values: None,
-            tick_text: None,
-            ticks: None,
-            ticks_on: None,
-            mirror: None,
-            tick_length: None,
-            tick_width: None,
-            tick_color: None,
-            show_tick_labels: None,
-            auto_margin: None,
-            show_spikes: None,
-            spike_color: None,
-            spike_thickness: None,
-            spike_dash: None,
-            spike_mode: None,
-            spike_snap: None,
-            tick_font: None,
-            tick_angle: None,
-            tick_prefix: None,
-            show_tick_prefix: None,
-            tick_suffix: None,
-            show_tick_suffix: None,
-            show_exponent: None,
-            exponent_format: None,
-            separate_thousands: None,
-            tick_format: None,
-            tick_format_stops: None,
-            hover_format: None,
-            show_line: None,
-            line_color: None,
-            line_width: None,
-            show_grid: None,
-            grid_color: None,
-            grid_width: None,
-            zero_line: None,
-            zero_line_color: None,
-            zero_line_width: None,
-            show_dividers: None,
-            divider_color: None,
-            divider_width: None,
-            side: None,
-            domain: None,
-            position: None,
-            calendar: None,
-        }
+        Default::default()
     }
 
     pub fn visible(mut self, visible: bool) -> Axis {
@@ -649,7 +835,7 @@ impl Axis {
     }
 
     pub fn color<C: Color>(mut self, color: C) -> Axis {
-        self.color = Some(color.to_color_string());
+        self.color = Some(color.to_color());
         self
     }
 
@@ -673,8 +859,9 @@ impl Axis {
         self
     }
 
-    pub fn range(mut self, range: Vec<f64>) -> Axis {
-        self.range = Some(range);
+    pub fn range<C: NumOrString>(mut self, range: Vec<C>) -> Axis {
+        let wrapped = to_num_or_string_wrapper(range);
+        self.range = Some(wrapped);
         self
     }
 
@@ -749,7 +936,7 @@ impl Axis {
     }
 
     pub fn tick_color<C: Color>(mut self, tick_color: C) -> Axis {
-        self.tick_color = Some(tick_color.to_color_string());
+        self.tick_color = Some(tick_color.to_color());
         self
     }
 
@@ -769,7 +956,7 @@ impl Axis {
     }
 
     pub fn spike_color<C: Color>(mut self, spike_color: C) -> Axis {
-        self.spike_color = Some(spike_color.to_color_string());
+        self.spike_color = Some(spike_color.to_color());
         self
     }
 
@@ -843,7 +1030,7 @@ impl Axis {
         self
     }
 
-    pub fn tick_format_stops(mut self, tick_format_stops: TickFormatStops) -> Axis {
+    pub fn tick_format_stops(mut self, tick_format_stops: Vec<TickFormatStop>) -> Axis {
         self.tick_format_stops = Some(tick_format_stops);
         self
     }
@@ -859,7 +1046,7 @@ impl Axis {
     }
 
     pub fn line_color<C: Color>(mut self, line_color: C) -> Axis {
-        self.line_color = Some(line_color.to_color_string());
+        self.line_color = Some(line_color.to_color());
         self
     }
 
@@ -874,7 +1061,7 @@ impl Axis {
     }
 
     pub fn grid_color<C: Color>(mut self, grid_color: C) -> Axis {
-        self.grid_color = Some(grid_color.to_color_string());
+        self.grid_color = Some(grid_color.to_color());
         self
     }
 
@@ -889,7 +1076,7 @@ impl Axis {
     }
 
     pub fn zero_line_color<C: Color>(mut self, zero_line_color: C) -> Axis {
-        self.zero_line_color = Some(zero_line_color.to_color_string());
+        self.zero_line_color = Some(zero_line_color.to_color());
         self
     }
 
@@ -904,7 +1091,7 @@ impl Axis {
     }
 
     pub fn divider_color<C: Color>(mut self, divider_color: C) -> Axis {
-        self.divider_color = Some(divider_color.to_color_string());
+        self.divider_color = Some(divider_color.to_color());
         self
     }
 
@@ -913,13 +1100,23 @@ impl Axis {
         self
     }
 
+    pub fn anchor(mut self, anchor: &str) -> Axis {
+        self.anchor = Some(anchor.to_owned());
+        self
+    }
+
     pub fn side(mut self, side: Side) -> Axis {
         self.side = Some(side);
         self
     }
 
-    pub fn domain(mut self, domain: Vec<f64>) -> Axis {
-        self.domain = Some(domain);
+    pub fn overlaying(mut self, overlaying: &str) -> Axis {
+        self.overlaying = Some(overlaying.to_owned());
+        self
+    }
+
+    pub fn domain(mut self, domain: &[f64]) -> Axis {
+        self.domain = Some(domain.to_vec());
         self
     }
 
@@ -928,6 +1125,16 @@ impl Axis {
         self
     }
 
+    pub fn range_slider(mut self, slider: RangeSlider) -> Axis {
+        self.range_slider = Some(slider);
+        self
+    }
+
+    pub fn range_selector(mut self, range_selector: RangeSelector) -> Axis {
+        self.range_selector = Some(range_selector);
+        self
+    }
+
     pub fn calendar(mut self, calendar: Calendar) -> Axis {
         self.calendar = Some(calendar);
         self
@@ -936,45 +1143,45 @@ impl Axis {
 
 #[derive(Serialize, Debug)]
 pub enum RowOrder {
-    #[serde(rename="top to bottom")]
+    #[serde(rename = "top to bottom")]
     TopToBottom,
-    #[serde(rename="bottom to top")]
+    #[serde(rename = "bottom to top")]
     BottomToTop,
 }
 
 #[derive(Serialize, Debug)]
 pub enum GridPattern {
-    #[serde(rename="independent")]
+    #[serde(rename = "independent")]
     Independent,
-    #[serde(rename="independent")]
+    #[serde(rename = "independent")]
     Coupled,
 }
 
 #[derive(Serialize, Debug)]
 pub enum GridXSide {
-    #[serde(rename="bottom")]
+    #[serde(rename = "bottom")]
     Bottom,
-    #[serde(rename="bottom plot")]
+    #[serde(rename = "bottom plot")]
     BottomPlot,
-    #[serde(rename="top plot")]
+    #[serde(rename = "top plot")]
     TopPlot,
-    #[serde(rename="top")]
+    #[serde(rename = "top")]
     Top,
 }
 
 #[derive(Serialize, Debug)]
 pub enum GridYSide {
-    #[serde(rename="left")]
+    #[serde(rename = "left")]
     Left,
-    #[serde(rename="left plot")]
+    #[serde(rename = "left plot")]
     LeftPlot,
-    #[serde(rename="right plot")]
+    #[serde(rename = "right plot")]
     RightPlot,
-    #[serde(rename="right")]
+    #[serde(rename = "right")]
     Right,
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct GridDomain {
     #[serde(skip_serializing_if = "Option::is_none")]
     x: Option<Vec<f64>>,
@@ -984,7 +1191,7 @@ pub struct GridDomain {
 
 impl GridDomain {
     pub fn new() -> GridDomain {
-        GridDomain {x: None, y: None}
+        Default::default()
     }
 
     pub fn x(mut self, x: Vec<f64>) -> GridDomain {
@@ -998,50 +1205,37 @@ impl GridDomain {
     }
 }
 
-#[derive(Serialize, Debug)]
+#[derive(Serialize, Debug, Default)]
 pub struct LayoutGrid {
     #[serde(skip_serializing_if = "Option::is_none")]
     rows: Option<usize>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="roworder")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "roworder")]
     row_order: Option<RowOrder>,
     #[serde(skip_serializing_if = "Option::is_none")]
     columns: Option<usize>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="subplots")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "subplots")]
     sub_plots: Option<Vec<String>>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="xaxes")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxes")]
     x_axes: Option<Vec<String>>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="yaxes")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxes")]
     y_axes: Option<Vec<String>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pattern: Option<GridPattern>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="xgap")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xgap")]
     x_gap: Option<f64>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="ygap")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "ygap")]
     y_gap: Option<f64>,
     #[serde(skip_serializing_if = "Option::is_none")]
     domain: Option<GridDomain>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="xside")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xside")]
     x_side: Option<GridXSide>,
-    #[serde(skip_serializing_if = "Option::is_none", rename="yside")]
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yside")]
     y_side: Option<GridYSide>,
 }
 
 impl LayoutGrid {
     pub fn new() -> LayoutGrid {
-        LayoutGrid {
-            rows: None,
-            row_order: None,
-            columns: None,
-            sub_plots: None,
-            x_axes: None,
-            y_axes: None,
-            pattern: None,
-            x_gap: None,
-            y_gap: None,
-            domain: None,
-            x_side: None,
-            y_side: None,
-        }
+        Default::default()
     }
 
     pub fn rows(mut self, rows: usize) -> LayoutGrid {
@@ -1105,6 +1299,94 @@ impl LayoutGrid {
 }
 
 #[derive(Serialize, Debug)]
+pub enum UniformTextMode {
+    #[serde(rename = "false")]
+    False,
+    #[serde(rename = "hide")]
+    Hide,
+    #[serde(rename = "show")]
+    Show,
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct UniformText {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    mode: Option<TruthyEnum<UniformTextMode>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    min_size: Option<usize>,
+}
+
+impl UniformText {
+    pub fn new() -> UniformText {
+        Default::default()
+    }
+
+    pub fn mode(mut self, mode: UniformTextMode) -> UniformText {
+        self.mode = Some(TruthyEnum { e: mode });
+        self
+    }
+
+    pub fn min_size(mut self, min_size: usize) -> UniformText {
+        self.min_size = Some(min_size);
+        self
+    }
+}
+
+#[derive(Serialize, Debug)]
+pub enum HoverMode {
+    #[serde(rename = "x")]
+    X,
+    #[serde(rename = "y")]
+    Y,
+    #[serde(rename = "closest")]
+    Closest,
+    #[serde(rename = "false")]
+    False,
+    #[serde(rename = "x unified")]
+    XUnified,
+    #[serde(rename = "y unified")]
+    YUnified,
+}
+
+#[derive(Serialize, Debug, Default)]
+pub struct ModeBar {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    orientation: Option<Orientation>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "bgcolor")]
+    background_color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    color: Option<ColorWrapper>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "activecolor")]
+    active_color: Option<ColorWrapper>,
+}
+
+impl ModeBar {
+    pub fn new() -> ModeBar {
+        Default::default()
+    }
+
+    pub fn orientation<C: Color>(mut self, orientation: Orientation) -> ModeBar {
+        self.orientation = Some(orientation);
+        self
+    }
+
+    pub fn background_color<C: Color>(mut self, background_color: C) -> ModeBar {
+        self.background_color = Some(background_color.to_color());
+        self
+    }
+
+    pub fn color<C: Color>(mut self, color: C) -> ModeBar {
+        self.color = Some(color.to_color());
+        self
+    }
+
+    pub fn active_color<C: Color>(mut self, active_color: C) -> ModeBar {
+        self.active_color = Some(active_color.to_color());
+        self
+    }
+}
+
+#[derive(Serialize, Debug, Default)]
 pub struct Layout {
     #[serde(skip_serializing_if = "Option::is_none")]
     title: Option<Title>,
@@ -1122,23 +1404,24 @@ pub struct Layout {
     height: Option<usize>,
     #[serde(skip_serializing_if = "Option::is_none")]
     font: Option<Font>,
-
-    // uniform_text: Option<UniformText>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    uniform_text: Option<UniformText>,
     #[serde(skip_serializing_if = "Option::is_none")]
     separators: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "paper_bgcolor")]
-    paper_background_color: Option<String>,
+    paper_background_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "plot_bgcolor")]
-    plot_background_color: Option<String>,
+    plot_background_color: Option<ColorWrapper>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "colorscale")]
     color_scale: Option<LayoutColorScale>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    colorway: Option<Vec<String>>,
+    colorway: Option<Vec<ColorWrapper>>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "coloraxis")]
     color_axis: Option<ColorAxis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "modebar")]
+    mode_bar: Option<ModeBar>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "hovermode")]
-    // mode_bar: Option<ModeBar>,
-    hover_mode: Option<String>,
+    hover_mode: Option<TruthyEnum<HoverMode>>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "clickmode")]
     click_mode: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "dragmode")]
@@ -1159,10 +1442,39 @@ pub struct Layout {
     grid: Option<LayoutGrid>,
     #[serde(skip_serializing_if = "Option::is_none")]
     calendar: Option<Calendar>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    xaxis: Option<Axis>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    yaxis: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis")]
+    x_axis: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis")]
+    y_axis: Option<Axis>,
+
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis2")]
+    x_axis2: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis2")]
+    y_axis2: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis3")]
+    x_axis3: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis3")]
+    y_axis3: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis4")]
+    x_axis4: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis4")]
+    y_axis4: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis5")]
+    x_axis5: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis5")]
+    y_axis5: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis6")]
+    x_axis6: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis6")]
+    y_axis6: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis7")]
+    x_axis7: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis7")]
+    y_axis7: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "xaxis8")]
+    x_axis8: Option<Axis>,
+    #[serde(skip_serializing_if = "Option::is_none", rename = "yaxis8")]
+    y_axis8: Option<Axis>,
 
     // ternary: Option<LayoutTernary>,
     // scene: Option<LayoutScene>,
@@ -1201,67 +1513,22 @@ pub struct Layout {
     waterfall_group_gap: Option<f64>,
 
     #[serde(skip_serializing_if = "Option::is_none", rename = "piecolorway")]
-    pie_colorway: Option<Vec<String>>,
+    pie_colorway: Option<Vec<ColorWrapper>>,
     #[serde(skip_serializing_if = "Option::is_none", rename = "extendpiecolors")]
     extend_pie_colors: Option<bool>,
 
     #[serde(skip_serializing_if = "Option::is_none", rename = "sunburstcolorway")]
-    sunburst_colorway: Option<Vec<String>>,
-    #[serde(skip_serializing_if = "Option::is_none", rename = "extendsuburstcolors")]
+    sunburst_colorway: Option<Vec<ColorWrapper>>,
+    #[serde(
+        skip_serializing_if = "Option::is_none",
+        rename = "extendsuburstcolors"
+    )]
     extend_sunburst_colors: Option<bool>,
 }
 
 impl Layout {
     pub fn new() -> Layout {
-        Layout {
-            title: None,
-            show_legend: None,
-            legend: None,
-            margin: None,
-            auto_size: None,
-            width: None,
-            height: None,
-            font: None,
-            separators: None,
-            paper_background_color: None,
-            plot_background_color: None,
-            color_scale: None,
-            colorway: None,
-            color_axis: None,
-            hover_mode: None,
-            click_mode: None,
-            drag_mode: None,
-            select_direction: None,
-            hover_distance: None,
-            spike_distance: None,
-            hover_label: None,
-            grid: None,
-            calendar: None,
-            xaxis: None,
-            yaxis: None,
-            template: None,
-            box_mode: None,
-            box_gap: None,
-            box_group_gap: None,
-            bar_mode: None,
-            bar_norm: None,
-            bar_gap: None,
-            bar_group_gap: None,
-
-            violin_mode: None,
-            violin_gap: None,
-            violin_group_gap: None,
-
-            waterfall_mode: None,
-            waterfall_gap: None,
-            waterfall_group_gap: None,
-
-            pie_colorway: None,
-            extend_pie_colors: None,
-
-            sunburst_colorway: None,
-            extend_sunburst_colors: None,
-        }
+        Default::default()
     }
 
     pub fn title(mut self, title: Title) -> Layout {
@@ -1304,18 +1571,23 @@ impl Layout {
         self
     }
 
+    pub fn uniform_text(mut self, uniform_text: UniformText) -> Layout {
+        self.uniform_text = Some(uniform_text);
+        self
+    }
+
     pub fn separators(mut self, separators: &str) -> Layout {
         self.separators = Some(separators.to_owned());
         self
     }
 
     pub fn paper_background_color<C: Color>(mut self, paper_background_color: C) -> Layout {
-        self.paper_background_color = Some(paper_background_color.to_color_string());
+        self.paper_background_color = Some(paper_background_color.to_color());
         self
     }
 
     pub fn plot_background_color<C: Color>(mut self, plot_background_color: C) -> Layout {
-        self.plot_background_color = Some(plot_background_color.to_color_string());
+        self.plot_background_color = Some(plot_background_color.to_color());
         self
     }
 
@@ -1335,8 +1607,23 @@ impl Layout {
         self
     }
 
-    pub fn hover_mode(mut self, hover_mode: &str) -> Layout {
-        self.hover_mode = Some(hover_mode.to_owned());
+    pub fn mode_bar(mut self, mode_bar: ModeBar) -> Layout {
+        self.mode_bar = Some(mode_bar);
+        self
+    }
+
+    /// Determines the mode of hover interactions. If "closest", a single hoverlabel will appear for the "closest"
+    /// point within the `hoverdistance`. If "x" (or "y"), multiple hoverlabels will appear for multiple points at
+    /// the "closest" x- (or y-) coordinate within the `hoverdistance`, with the caveat that no more than one hoverlabel
+    /// will appear per trace. If "x unified" (or "y unified"), a single hoverlabel will appear multiple points at
+    /// the closest x- (or y-) coordinate within the `hoverdistance` with the caveat that no more than one hoverlabel
+    /// will appear per trace. In this mode, spikelines are enabled by default perpendicular to the specified axis.
+    /// If false, hover interactions are disabled. If `clickmode` includes the "select" flag, `hovermode` defaults to
+    /// "closest". If `clickmode` lacks the "select" flag, it defaults to "x" or "y"
+    /// (depending on the trace's `orientation` value) for plots based on cartesian coordinates. For anything
+    /// else the default value is "closest".
+    pub fn hover_mode(mut self, hover_mode: HoverMode) -> Layout {
+        self.hover_mode = Some(TruthyEnum { e: hover_mode });
         self
     }
 
@@ -1380,13 +1667,83 @@ impl Layout {
         self
     }
 
-    pub fn xaxis(mut self, xaxis: Axis) -> Layout {
-        self.xaxis = Some(xaxis);
+    pub fn x_axis(mut self, xaxis: Axis) -> Layout {
+        self.x_axis = Some(xaxis);
         self
     }
 
-    pub fn yaxis(mut self, yaxis: Axis) -> Layout {
-        self.yaxis = Some(yaxis);
+    pub fn y_axis(mut self, yaxis: Axis) -> Layout {
+        self.y_axis = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis2(mut self, xaxis: Axis) -> Layout {
+        self.x_axis2 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis2(mut self, yaxis: Axis) -> Layout {
+        self.y_axis2 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis3(mut self, xaxis: Axis) -> Layout {
+        self.x_axis3 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis3(mut self, yaxis: Axis) -> Layout {
+        self.y_axis3 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis4(mut self, xaxis: Axis) -> Layout {
+        self.x_axis4 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis4(mut self, yaxis: Axis) -> Layout {
+        self.y_axis4 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis5(mut self, xaxis: Axis) -> Layout {
+        self.x_axis5 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis5(mut self, yaxis: Axis) -> Layout {
+        self.y_axis5 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis6(mut self, xaxis: Axis) -> Layout {
+        self.x_axis6 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis6(mut self, yaxis: Axis) -> Layout {
+        self.y_axis6 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis7(mut self, xaxis: Axis) -> Layout {
+        self.x_axis7 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis7(mut self, yaxis: Axis) -> Layout {
+        self.y_axis7 = Some(yaxis);
+        self
+    }
+
+    pub fn x_axis8(mut self, xaxis: Axis) -> Layout {
+        self.x_axis8 = Some(xaxis);
+        self
+    }
+
+    pub fn y_axis8(mut self, yaxis: Axis) -> Layout {
+        self.y_axis8 = Some(yaxis);
         self
     }
 
